@@ -39,116 +39,26 @@ std::istream& operator>>( std::istream& is, tChannelID& ret )
 
 namespace vdrlive {
 
-template<class T>
-  void AppendHtmlEscapedAndCorrectNonUTF8(T &target, const char* s, const char *end, bool tooltip){
-// append c-string s to target, HTML escape some characters
-// replace invalid UTF8 characters with ?
-  if(!s) return;
-  if (!end) end = s + strlen(s);
-  int l = 0;                    // length of current UTF8 codepoint
-  size_t i = 0;                 // number of not yet appended chars
-  const char* notAppended = s;  // position of the first character which is not yet appended
-  for (const char* current = s; *current && current < end; current+=l) {
-  l = utf8CodepointIsValid(current);
-    switch(l) {
-      case 1:
-        switch(*current) {
-          case '&':  target.append(notAppended, i); target.append("&amp;");  notAppended = notAppended + i + 1; i = 0;   break;
-          case '\"': target.append(notAppended, i); target.append("&quot;"); notAppended = notAppended + i + 1; i = 0;   break;
-          case '\'': target.append(notAppended, i); target.append("&apos;"); notAppended = notAppended + i + 1; i = 0;   break;
-          case '<':  target.append(notAppended, i); target.append("&lt;");   notAppended = notAppended + i + 1; i = 0;   break;
-          case '>':  target.append(notAppended, i); target.append("&gt;");   notAppended = notAppended + i + 1; i = 0;   break;
-          case 10:
-          case 13:
-              target.append(notAppended, i); target.append("&lt;br/&gt;");   notAppended = notAppended + i + 1; i = 0;   break;
-          default:   i++; break;
-          }
-        break;
-      case 2:
-      case 3:
-      case 4:
-        i += l;
-        break;
-      default:
-// invalid UTF8
-        target.append(notAppended, i);
-        target.append("?");
-        notAppended = notAppended + i + 1;
-        i = 0;
-        l = 1;
-      }
-    }
-    target.append(notAppended, i);
-  }
-template void AppendHtmlEscapedAndCorrectNonUTF8<std::string>(std::string &target, const char* s, const char *end, bool tooltip);
-template void AppendHtmlEscapedAndCorrectNonUTF8<cToSvConcat<0>>(cToSvConcat<0> &target, const char* s, const char *end, bool tooltip);
-
-template<typename T>
-  void AppendDuration(T &target, char const* format, int duration)
-  {
-    int minutes = (duration + 30) / 60;
-    int hours = minutes / 60;
-    minutes %= 60;
-    stringAppendFormated(target, format, hours, minutes);
-  }
-template void AppendDuration<std::string>(std::string &target, char const* format, int duration);
-template void AppendDuration<cToSvConcat<0>>(cToSvConcat<0> &target, char const* format, int duration);
-
   std::string FormatDuration(char const* format, int duration)
   {
-    std::string result;
+    cToSvConcat result;
     AppendDuration(result, format, duration);
-    return result;
+    return std::string(result);
   }
 
-  std::string StringReplace(cSv text, cSv substring, cSv replacement)
-  {
-    std::string result(text);
-    size_t pos = 0;
-    while ( ( pos = result.find( substring, pos ) ) != std::string::npos ) {
-      result.replace( pos, substring.length(), replacement );
-      pos += replacement.length();
-    }
-    return result;
-  }
-
-  std::vector<std::string> StringSplit(cSv text, char delimiter )
-  {
-    std::vector<std::string> result;
-    size_t last = 0, pos;
-    while ( ( pos = text.find( delimiter, last ) ) != std::string::npos ) {
-      result.emplace_back( text.substr( last, pos - last ) );
-      last = pos + 1;
-    }
-    if ( last < text.length() )
-      result.emplace_back( text.substr( last ) );
-    return result;
-  }
-
-  cSv StringWordTruncate(cSv input, size_t maxLen, bool& truncated)
-  {
-    if (input.length() <= maxLen)
-    {
+  cSv StringWordTruncate(cSv text, size_t maxLen, bool& truncated) {
+// Return text truncted to maxLen characters. If such truncation is required,
+// truncate at any char in the char list in find_last_of (see code below ...)
+    if (text.length() <= maxLen) {
       truncated = false;
-      return input;
+      return text;
     }
     truncated = true;
-    cSv result = input.substr(0, maxLen);
-    size_t pos = result.find_last_of(" \t,;:.\n?!'\"/\\()[]{}*+-");
-    return result.substr(0, pos);
-  }
-
-  std::string StringFormatBreak(cSv input)
-  {
-    return StringReplace(input, "\n", "<br/>" );
-  }
-
-  std::string StringEscapeAndBreak(cSv input, const char* nl)
-  {
-    std::stringstream plainBuilder;
-    HtmlEscOstream builder(plainBuilder);  // see https://web.archive.org/web/20151208133551/http://www.tntnet.org/apidoc_master/html/classtnt_1_1HtmlEscOstream.html
-    builder << input;
-    return StringReplace(plainBuilder.str(), "\n", nl);
+    cSv result = text.substr(0, maxLen);
+    size_t pos = result.find_last_of(" \t\n,;:.?!()[]{}*+-");
+    if (pos == std::string::npos) return result;
+    if (strchr("?!)]}", result[pos])) ++pos; // truncate after these characters
+    return StringTrim(result.substr(0, pos));
   }
 
   cSv StringTrim(cSv str)
@@ -161,47 +71,6 @@ template void AppendDuration<cToSvConcat<0>>(cToSvConcat<0> &target, char const*
     return trailBlankRemoved.substr(pos);
   }
 
-// Spielfilm Thailand / Deutschland / Großbritannien 2015 (Rak ti Khon Kaen)
-#define MAX_LEN_ST 70
-template<class T>
-  void AppendTextMaxLen(T &target, const char *text) {
-// append text to target, but
-//   stop at line break in text (10 || 13)
-//   only up to MAX_LEN_ST characters. If such truncation is required, truncate at ' '
-
-// escape HTML characters, and correct invalid utf8
-    if (!text || !*text ) return;
-    int len = strlen(text);
-    int lb = len;
-    for (const char *s = text; *s; s++) if (*s == 10 || *s == 13) { lb = s-text; break;}
-    if (len < MAX_LEN_ST && lb == len)
-      AppendHtmlEscapedAndCorrectNonUTF8(target, text);
-    else if (lb < MAX_LEN_ST) {
-      AppendHtmlEscapedAndCorrectNonUTF8(target, text, text + lb);
-      target.append(" ...");
-    } else {
-      const char *end = text + MAX_LEN_ST;
-      for (; *end && *end != ' ' && *end != 10 && *end != 13; end++);
-      AppendHtmlEscapedAndCorrectNonUTF8(target, text, end);
-      if (*end) target.append(" ...");
-    }
-  }
-template void AppendTextMaxLen<std::string>(std::string &target, const char *text);
-template void AppendTextMaxLen<cToSvConcat<0>>(cToSvConcat<0> &target, const char *text);
-
-template<class T>
-  void AppendTextTruncateOnWord(T &target, const char *text, int max_len, bool tooltip) {
-// append text to target, but only up to max_len characters. If such truncation is required, truncate at ' '
-// escape HTML characters, and correct invalid UTF8
-    if (!text || !*text ) return;
-    const char *end = text + std::min((int)strlen(text), max_len);
-    for (; *end && *end != ' '; end++);
-    AppendHtmlEscapedAndCorrectNonUTF8(target, text, end, tooltip);
-    if (*end) target.append(" ...");
-  }
-template void AppendTextTruncateOnWord<std::string>(std::string &target, const char *text, int max_len, bool tooltip);
-template void AppendTextTruncateOnWord<cToSvConcat<0>>(cToSvConcat<0> &target, const char *text, int max_len, bool tooltip);
-
   std::string MD5Hash(std::string const& str)
   {
     char* szInput = strdup(str.c_str());
@@ -209,16 +78,10 @@ template void AppendTextTruncateOnWord<cToSvConcat<0>>(cToSvConcat<0> &target, c
     char* szRes = MD5String(szInput);
     std::string res = szRes;
     free(szRes);
+    free(szInput);
     return res;
   }
 
-  std::string xxHash32(cSv str)
-  {
-    char res[8];
-    stringhelpers_internal::addCharsHex(res, 8, XXH32(str.data(), str.length(), 20) );
-    return std::string(res, 8);
-  }
-
   XXH64_hash_t parse_hex_64(cSv str) {
     if (str.length() != 16) {
       esyslog("live: ERROR in parse_hex_64, hex = \"%.*s\" is not 16 chars long", (int)str.length(), str.data());
@@ -277,10 +140,9 @@ template void AppendTextTruncateOnWord<cToSvConcat<0>>(cToSvConcat<0> &target, c
     return timestring;
   }
 
-  time_t GetTimeT(std::string timestring) // timestring in HH:MM
+  time_t GetTimeT(cSv timestring) // timestring in HH:MM
   {
-    timestring = StringReplace(timestring, ":", "");
-    int iTime = parse_int<int>( timestring );
+    int iTime = parse_int<int>( cToSvReplace(timestring, ":", "") );
     struct tm tm_r;
     time_t t = time(NULL);
     tm* tmnow = localtime_r(&t, &tm_r);
@@ -289,42 +151,6 @@ template void AppendTextTruncateOnWord<cToSvConcat<0>>(cToSvConcat<0> &target, c
     return mktime(tmnow);
   }
 
-  struct urlencoder
-  {
-    std::ostream& ostr_;
-    explicit urlencoder( std::ostream& ostr ): ostr_( ostr ) {}
-
-    void operator()( char ch )
-    {
-      static const char allowedChars[] = {
-        //    0 ,  1 ,  2 ,  3 ,  4 ,  5 ,  6 ,  7 ,  8 ,  9 ,  A ,  B ,  C ,  D ,  E ,  F ,
-        '_', '_', '_', '_', '_', '_', '_', '_', '_', '_', '_', '_', '_', '_', '_', '_',  //00
-        '_', '_', '_', '_', '_', '_', '_', '_', '_', '_', '_', '_', '_', '_', '_', '_',  //10
-        '_', '_', '_', '_', '_', '_', '_', '_', '_', '_', '_', '_', '_', 0x2D,0x2E,0x2F,//20
-        0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,'_', '_', '_', '_', '_', '_',  //30
-        '_', 0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4A,0x4B,0x4C,0x4D,0x4E,0x4F,//40
-        0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5A,'_', '_', '_', '_', '_',  //50
-        '_', 0x61,0x62,0x63,0x64,0x65,0x66,0x67,0x68,0x69,0x6A,0x6B,0x6C,0x6D,0x6E,0x6F,//60
-        0x70,0x71,0x72,0x73,0x74,0x75,0x76,0x77,0x78,0x79,0x7A,'_', '_', '_', '_', '_'   //70
-        // everything above 127 (for signed char, below zero) is replaced with '_'
-      };
-
-      if ( ch == ' ' )
-        ostr_ << '+';
-      else if ( static_cast<signed char>(ch) < 0 || allowedChars[ size_t( ch ) ] == '_' )
-        ostr_ << '%' << std::setw( 2 ) << std::setfill( '0' ) << std::hex << int( static_cast<unsigned char>(ch) );
-      else
-        ostr_ << ch;
-    }
-  };
-
-  std::string StringUrlEncode(cSv input)
-  {
-    std::stringstream ostr;
-    std::for_each (input.begin(), input.end(), urlencoder( ostr ) );
-    return ostr.str();
-  }
-
 // return the time value as time_t from <datestring> formatted with <format>
   time_t GetDateFromDatePicker(cSv datestring, cSv format)
   {
@@ -346,11 +172,11 @@ template void AppendTextTruncateOnWord<cToSvConcat<0>>(cToSvConcat<0> &target, c
   std::string DatePickerToC(time_t date, cSv format)
   {
     if (date == 0) return "";
-    std::string cformat(format);
-    cformat = StringReplace(cformat, "mm", "%m");
-    cformat = StringReplace(cformat, "dd", "%d");
-    cformat = StringReplace(cformat, "yyyy", "%Y");
-    return std::string(cToSvDateTime(cformat.c_str(), date));
+    cToSvConcat cformat;
+    cformat.appendReplace(format, "mm", "%m");
+    cformat.replaceAll("dd", "%d");
+    cformat.replaceAll("yyyy", "%Y");
+    return std::string(cToSvDateTime(cformat, date));
   }
   int timeStringToInt(const char *t) {
   // input: t in xx:xx format
